@@ -89,7 +89,7 @@ class FirewallClientZoneSettings(object):
             self.settings = settings
         else:
             self.settings = ["", "", "", False, DEFAULT_ZONE_TARGET, [], [],
-                             [], False, [], [], [], [], []]
+                             [], False, [], [], [], [], [], [], False]
 
     @handle_exceptions
     def __repr__(self):
@@ -193,6 +193,30 @@ class FirewallClientZoneSettings(object):
     def queryProtocol(self, protocol):
         return protocol in self.settings[13]
 
+    @handle_exceptions
+    def getSourcePorts(self):
+        return self.settings[14]
+    @handle_exceptions
+    def setSourcePorts(self, ports):
+        self.settings[14] = ports
+    @handle_exceptions
+    def addSourcePort(self, port, protocol):
+        if (port,protocol) not in self.settings[14]:
+            self.settings[14].append((port,protocol))
+        else:
+            raise FirewallError(errors.ALREADY_ENABLED,
+                                "'%s:%s'" % (port, protocol))
+    @handle_exceptions
+    def removeSourcePort(self, port, protocol):
+        if (port,protocol) in self.settings[14]:
+            self.settings[14].remove((port,protocol))
+        else:
+            raise FirewallError(errors.NOT_ENABLED,
+                                "'%s:%s'" % (port, protocol))
+    @handle_exceptions
+    def querySourcePort(self, port, protocol):
+        return (port,protocol) in self.settings[14]
+
     @handle_exceptions
     def getIcmpBlocks(self):
         return sorted(self.settings[7])
@@ -215,12 +239,55 @@ class FirewallClientZoneSettings(object):
     def queryIcmpBlock(self, icmptype):
         return icmptype in self.settings[7]
 
+    @handle_exceptions
+    def getIcmpBlockInversion(self):
+        return self.settings[15]
+    @handle_exceptions
+    def setIcmpBlockInversion(self, flag):
+        self.settings[15] = flag
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def addIcmpBlockInversion(self):
+        if not self.settings[15]:
+            self.settings[15] = True
+        else:
+            FirewallError(errors.ALREADY_ENABLED, "icmp-block-inversion")
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def removeIcmpBlockInversion(self):
+        if self.settings[15]:
+            self.settings[15] = False
+        else:
+            FirewallError(errors.NOT_ENABLED, "icmp-block-inversion")
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def queryIcmpBlockInversion(self):
+        return self.settings[15]
+
     @handle_exceptions
     def getMasquerade(self):
         return self.settings[8]
     @handle_exceptions
     def setMasquerade(self, masquerade):
         self.settings[8] = masquerade
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def addMasquerade(self):
+        if not self.settings[8]:
+            self.settings[8] = True
+        else:
+            FirewallError(errors.ALREADY_ENABLED, "masquerade")
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def removeMasquerade(self):
+        if self.settings[8]:
+            self.settings[8] = False
+        else:
+            FirewallError(errors.NOT_ENABLED, "masquerade")
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def queryMasquerade(self):
+        return self.settings[8]
 
     @handle_exceptions
     def getForwardPorts(self):
@@ -517,6 +584,33 @@ class FirewallClientConfigZone(object):
     def queryProtocol(self, protocol):
         return self.fw_zone.queryProtocol(protocol)
 
+    # source-port
+
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def getSourcePorts(self):
+        return self.fw_zone.getSourcePorts()
+
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def setSourcePorts(self, ports):
+        self.fw_zone.setSourcePorts(ports)
+
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def addSourcePort(self, port, protocol):
+        self.fw_zone.addSourcePort(port, protocol)
+
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def removeSourcePort(self, port, protocol):
+        self.fw_zone.removeSourcePort(port, protocol)
+
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def querySourcePort(self, port, protocol):
+        return self.fw_zone.querySourcePort(port, protocol)
+
     # icmp block
 
     @slip.dbus.polkit.enable_proxy
@@ -544,6 +638,33 @@ class FirewallClientConfigZone(object):
     def queryIcmpBlock(self, icmptype):
         return self.fw_zone.queryIcmpBlock(icmptype)
 
+    # icmp-block-inversion
+
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def getIcmpBlockInversion(self):
+        return self.fw_zone.getIcmpBlockInversion()
+
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def setIcmpBlockInversion(self, inversion):
+        self.fw_zone.setIcmpBlockInversion(inversion)
+
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def addIcmpBlockInversion(self):
+        self.fw_zone.addIcmpBlockInversion()
+
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def removeIcmpBlockInversion(self):
+        self.fw_zone.removeIcmpBlockInversion()
+
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def queryIcmpBlockInversion(self):
+        return self.fw_zone.queryIcmpBlockInversion()
+
     # masquerade
 
     @slip.dbus.polkit.enable_proxy
@@ -700,7 +821,7 @@ class FirewallClientServiceSettings(object):
         if settings:
             self.settings = settings
         else:
-            self.settings = ["", "", "", [], [], {}, []]
+            self.settings = ["", "", "", [], [], {}, [], []]
 
     @handle_exceptions
     def __repr__(self):
@@ -773,6 +894,30 @@ class FirewallClientServiceSettings(object):
     def queryProtocol(self, protocol):
         return protocol in self.settings[6]
 
+    @handle_exceptions
+    def getSourcePorts(self):
+        return self.settings[7]
+    @handle_exceptions
+    def setSourcePorts(self, ports):
+        self.settings[7] = ports
+    @handle_exceptions
+    def addSourcePort(self, port, protocol):
+        if (port,protocol) not in self.settings[7]:
+            self.settings[7].append((port,protocol))
+        else:
+            raise FirewallError(errors.ALREADY_ENABLED,
+                                "'%s:%s'" % (port, protocol))
+    @handle_exceptions
+    def removeSourcePort(self, port, protocol):
+        if (port,protocol) in self.settings[7]:
+            self.settings[7].remove((port,protocol))
+        else:
+            raise FirewallError(errors.NOT_ENABLED,
+                                "'%s:%s'" % (port, protocol))
+    @handle_exceptions
+    def querySourcePort(self, port, protocol):
+        return (port,protocol) in self.settings[7]
+
     @handle_exceptions
     def getModules(self):
         return sorted(self.settings[4])
@@ -810,8 +955,11 @@ class FirewallClientServiceSettings(object):
             raise FirewallError(errors.ALREADY_ENABLED, "'%s:%s'" % \
                                 (dest_type, address))
     @handle_exceptions
-    def removeDestination(self, dest_type):
+    def removeDestination(self, dest_type, address=None):
         if dest_type in self.settings[5]:
+            if address is not None and self.settings[5][dest_type] != address:
+                raise FirewallError(errors.NOT_ENABLED, "'%s:%s'" % \
+                                    (dest_type, address))
             del self.settings[5][dest_type]
         else:
             raise FirewallError(errors.NOT_ENABLED, "'%s'" % dest_type)
@@ -1176,6 +1324,33 @@ class FirewallClientConfigService(object):
     def queryProtocol(self, protocol):
         return self.fw_service.queryProtocol(protocol)
 
+    # source-port
+
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def getSourcePorts(self):
+        return self.fw_service.getSourcePorts()
+
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def setSourcePorts(self, ports):
+        self.fw_service.setSourcePorts(ports)
+
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def addSourcePort(self, port, protocol):
+        self.fw_service.addSourcePort(port, protocol)
+
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def removeSourcePort(self, port, protocol):
+        self.fw_service.removeSourcePort(port, protocol)
+
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def querySourcePort(self, port, protocol):
+        return self.fw_service.querySourcePort(port, protocol)
+
     # module
 
     @slip.dbus.polkit.enable_proxy
@@ -1227,7 +1402,10 @@ class FirewallClientConfigService(object):
 
     @slip.dbus.polkit.enable_proxy
     @handle_exceptions
-    def removeDestination(self, destination):
+    def removeDestination(self, destination, address=None):
+        if address is not None and self.getDestination(destination) != address:
+            raise FirewallError(errors.NOT_ENABLED, "'%s:%s'" % \
+                                (destination, address))
         self.fw_service.removeDestination(destination)
 
     @slip.dbus.polkit.enable_proxy
@@ -1865,6 +2043,11 @@ class FirewallClientConfig(object):
 
     # ipset
 
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def getIPSetNames(self):
+        return dbus_to_python(self.fw_config.getIPSetNames())
+
     @slip.dbus.polkit.enable_proxy
     @handle_exceptions
     def listIPSets(self):
@@ -1884,11 +2067,19 @@ class FirewallClientConfig(object):
     @slip.dbus.polkit.enable_proxy
     @handle_exceptions
     def addIPSet(self, name, settings):
-        path = self.fw_config.addIPSet(name, tuple(settings.settings))
+        if isinstance(settings, FirewallClientIPSetSettings):
+            path = self.fw_config.addIPSet(name, tuple(settings.settings))
+        else:
+            path = self.fw_config.addIPSet(name, tuple(settings))
         return FirewallClientConfigIPSet(self.bus, path)
 
     # zone
 
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def getZoneNames(self):
+        return dbus_to_python(self.fw_config.getZoneNames())
+
     @slip.dbus.polkit.enable_proxy
     @handle_exceptions
     def listZones(self):
@@ -1918,11 +2109,19 @@ class FirewallClientConfig(object):
     @slip.dbus.polkit.enable_proxy
     @handle_exceptions
     def addZone(self, name, settings):
-        path = self.fw_config.addZone(name, tuple(settings.settings))
+        if isinstance(settings, FirewallClientZoneSettings):
+            path = self.fw_config.addZone(name, tuple(settings.settings))
+        else:
+            path = self.fw_config.addZone(name, tuple(settings))
         return FirewallClientConfigZone(self.bus, path)
 
     # service
 
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def getServiceNames(self):
+        return dbus_to_python(self.fw_config.getServiceNames())
+
     @slip.dbus.polkit.enable_proxy
     @handle_exceptions
     def listServices(self):
@@ -1942,11 +2141,19 @@ class FirewallClientConfig(object):
     @slip.dbus.polkit.enable_proxy
     @handle_exceptions
     def addService(self, name, settings):
-        path = self.fw_config.addService(name, tuple(settings.settings))
+        if isinstance(settings, FirewallClientServiceSettings):
+            path = self.fw_config.addService(name, tuple(settings.settings))
+        else:
+            path = self.fw_config.addService(name, tuple(settings))
         return FirewallClientConfigService(self.bus, path)
 
     # icmptype
 
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def getIcmpTypeNames(self):
+        return dbus_to_python(self.fw_config.getIcmpTypeNames())
+
     @slip.dbus.polkit.enable_proxy
     @handle_exceptions
     def listIcmpTypes(self):
@@ -1966,7 +2173,10 @@ class FirewallClientConfig(object):
     @slip.dbus.polkit.enable_proxy
     @handle_exceptions
     def addIcmpType(self, name, settings):
-        path = self.fw_config.addIcmpType(name, tuple(settings.settings))
+        if isinstance(settings, FirewallClientIcmpTypeSettings):
+            path = self.fw_config.addIcmpType(name, tuple(settings.settings))
+        else:
+            path = self.fw_config.addIcmpType(name, tuple(settings))
         return FirewallClientConfigIcmpType(self.bus, path)
 
     @slip.dbus.polkit.enable_proxy
@@ -2036,6 +2246,8 @@ class FirewallClient(object):
             "service-removed": "ServiceRemoved",
             "port-added": "PortAdded",
             "port-removed": "PortRemoved",
+            "source-port-added": "SourcePortAdded",
+            "source-port-removed": "SourcePortRemoved",
             "protocol-added": "ProtocolAdded",
             "protocol-removed": "ProtocolRemoved",
             "masquerade-added": "MasqueradeAdded",
@@ -2044,6 +2256,8 @@ class FirewallClient(object):
             "forward-port-removed": "ForwardPortRemoved",
             "icmp-block-added": "IcmpBlockAdded",
             "icmp-block-removed": "IcmpBlockRemoved",
+            "icmp-block-inversion-added": "IcmpBlockInversionAdded",
+            "icmp-block-inversion-removed": "IcmpBlockInversionRemoved",
             "richrule-added": "RichRuleAdded",
             "richrule-removed": "RichRuleRemoved",
             "interface-added": "InterfaceAdded",
@@ -2582,9 +2796,9 @@ class FirewallClient(object):
     @handle_exceptions
     def addForwardPort(self, zone, port, protocol, toport, toaddr,
                        timeout=0):
-        if not toport:
+        if toport is None:
             toport = ""
-        if not toaddr:
+        if toaddr is None:
             toaddr = ""
         return dbus_to_python(self.fw_zone.addForwardPort(zone, port, protocol,
                                                           toport, toaddr,
@@ -2598,9 +2812,9 @@ class FirewallClient(object):
     @slip.dbus.polkit.enable_proxy
     @handle_exceptions
     def queryForwardPort(self, zone, port, protocol, toport, toaddr):
-        if not toport:
+        if toport is None:
             toport = ""
-        if not toaddr:
+        if toaddr is None:
             toaddr = ""
         return dbus_to_python(self.fw_zone.queryForwardPort(zone,
                                                             port, protocol,
@@ -2609,14 +2823,38 @@ class FirewallClient(object):
     @slip.dbus.polkit.enable_proxy
     @handle_exceptions
     def removeForwardPort(self, zone, port, protocol, toport, toaddr):
-        if not toport:
+        if toport is None:
             toport = ""
-        if not toaddr:
+        if toaddr is None:
             toaddr = ""
         return dbus_to_python(self.fw_zone.removeForwardPort(zone,
                                                              port, protocol,
                                                              toport, toaddr))
 
+    # source ports
+
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def addSourcePort(self, zone, port, protocol, timeout=0):
+        return dbus_to_python(self.fw_zone.addSourcePort(zone, port, protocol,
+                                                         timeout))
+
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def getSourcePorts(self, zone):
+        return dbus_to_python(self.fw_zone.getSourcePorts(zone))
+
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def querySourcePort(self, zone, port, protocol):
+        return dbus_to_python(self.fw_zone.querySourcePort(zone, port, protocol))
+
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def removeSourcePort(self, zone, port, protocol):
+        return dbus_to_python(self.fw_zone.removeSourcePort(zone, port,
+                                                            protocol))
+
     # icmpblock
 
     @slip.dbus.polkit.enable_proxy
@@ -2639,6 +2877,23 @@ class FirewallClient(object):
     def removeIcmpBlock(self, zone, icmp):
         return dbus_to_python(self.fw_zone.removeIcmpBlock(zone, icmp))
 
+    # icmp block inversion
+
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def addIcmpBlockInversion(self, zone):
+        return dbus_to_python(self.fw_zone.addIcmpBlockInversion(zone))
+
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def queryIcmpBlockInversion(self, zone):
+        return dbus_to_python(self.fw_zone.queryIcmpBlockInversion(zone))
+
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def removeIcmpBlockInversion(self, zone):
+        return dbus_to_python(self.fw_zone.removeIcmpBlockInversion(zone))
+
     # direct chain
 
     @slip.dbus.polkit.enable_proxy
