@@ -42,7 +42,7 @@ APPLET_NAME = 'firewall-applet'
 DATADIR = '/usr/share/' + DAEMON_NAME
 CONFIG_GLADE_NAME = CONFIG_NAME + '.glade'
 COPYRIGHT = '(C) 2010-2016 Red Hat, Inc.'
-VERSION = '0.4.1.2'
+VERSION = '0.4.3.2'
 AUTHORS = [
     "Thomas Woerner <twoerner@redhat.com>",
     "Jiri Popelka <jpopelka@redhat.com>",
@@ -85,6 +85,9 @@ FIREWALLD_DIRECT = ETC_FIREWALLD + '/direct.xml'
 
 LOCKDOWN_WHITELIST = ETC_FIREWALLD + '/lockdown-whitelist.xml'
 
+SYSCONFIGDIR = '/etc/conf.d'
+IFCFGDIR = SYSCONFIGDIR + '/network-scripts'
+
 SYSCTL_CONFIG = '/etc/sysctl.conf'
 
 # commands used by backends
