@@ -23,14 +23,13 @@ __all__ = [ "FirewallConfig" ]
 
 import copy
 import os, os.path
+import shutil
 from firewall import config
-from firewall.core.base import *
 from firewall.core.logger import log
 from firewall.core.io.icmptype import IcmpType, icmptype_reader, icmptype_writer
 from firewall.core.io.service import Service, service_reader, service_writer
 from firewall.core.io.zone import Zone, zone_reader, zone_writer
 from firewall.core.io.ipset import IPSet, ipset_reader, ipset_writer
-from firewall.functions import portStr
 from firewall import errors
 from firewall.errors import FirewallError
 
@@ -40,10 +39,12 @@ class FirewallConfig(object):
         self.__init_vars()
 
     def __repr__(self):
-        return '%s(%r, %r, %r, %r, %r, %r, %r, %r, %r, %r, %r)' % (self.__class__,
-                   self._ipsets, self._icmptypes, self._services, self._zones,
-                   self._builtin_ipsets, self._builtin_icmptypes, self._builtin_services, self._builtin_zones,
-                   self._firewalld_conf, self._policies, self._direct)
+        return '%s(%r, %r, %r, %r, %r, %r, %r, %r, %r, %r, %r)' % \
+            (self.__class__,
+             self._ipsets, self._icmptypes, self._services, self._zones,
+             self._builtin_ipsets, self._builtin_icmptypes,
+             self._builtin_services, self._builtin_zones,
+             self._firewalld_conf, self._policies, self._direct)
 
     def __init_vars(self):
         self._ipsets = { }
@@ -181,7 +182,7 @@ class FirewallConfig(object):
                                 "self._ipsets[%s] != obj" % obj.name)
         elif obj.name not in self._builtin_ipsets:
             raise FirewallError(errors.NO_DEFAULTS,
-                            "'%s' not in self._builtin_ipsets" % obj.name)
+                            "'%s' not a built-in ipset" % obj.name)
         self._remove_ipset(obj)
         return self._builtin_ipsets[obj.name]
 
@@ -205,12 +206,9 @@ class FirewallConfig(object):
             return obj
 
     def new_ipset(self, name, conf):
-        try:
-            self.get_ipset(name)
-        except:
-            pass
-        else:
-            raise FirewallError(errors.NAME_CONFLICT, "new_ipset(): '%s'" % name)
+        if name in self._ipsets:
+            raise FirewallError(errors.NAME_CONFLICT,
+                                "new_ipset(): '%s'" % name)
 
         x = IPSet()
         x.check_name(name)
@@ -297,13 +295,19 @@ class FirewallConfig(object):
 
     def _remove_ipset(self, obj):
         if obj.name not in self._ipsets:
-            raise FirewallError(errors.INVALID_IPSET,
-                                "'%s' not in self._ipsets" % obj.name)
+            raise FirewallError(errors.INVALID_IPSET, obj.name)
         if obj.path != config.ETC_FIREWALLD_IPSETS:
             raise FirewallError(errors.INVALID_DIRECTORY,
                                 "'%s' != '%s'" % (obj.path,
                                                   config.ETC_FIREWALLD_IPSETS))
-        os.remove("%s/%s.xml" % (obj.path, obj.name))
+
+        name = "%s/%s.xml" % (obj.path, obj.name)
+        try:
+            shutil.move(name, "%s.old" % name)
+        except Exception as msg:
+            log.error("Backup of file '%s' failed: %s", name, msg)
+            os.remove(name)
+
         del self._ipsets[obj.name]
 
     def check_builtin_ipset(self, obj):
@@ -351,7 +355,7 @@ class FirewallConfig(object):
                                 "self._icmptypes[%s] != obj" % obj.name)
         elif obj.name not in self._builtin_icmptypes:
             raise FirewallError(errors.NO_DEFAULTS,
-                            "'%s' not in self._builtin_icmptypes" % obj.name)
+                                "'%s' not a built-in icmptype" % obj.name)
         self._remove_icmptype(obj)
         return self._builtin_icmptypes[obj.name]
 
@@ -375,12 +379,9 @@ class FirewallConfig(object):
             return obj
 
     def new_icmptype(self, name, conf):
-        try:
-            self.get_icmptype(name)
-        except:
-            pass
-        else:
-            raise FirewallError(errors.NAME_CONFLICT, "new_icmptype(): '%s'" % name)
+        if name in self._icmptypes:
+            raise FirewallError(errors.NAME_CONFLICT,
+                                "new_icmptype(): '%s'" % name)
 
         x = IcmpType()
         x.check_name(name)
@@ -467,13 +468,19 @@ class FirewallConfig(object):
 
     def _remove_icmptype(self, obj):
         if obj.name not in self._icmptypes:
-            raise FirewallError(errors.INVALID_ICMPTYPE,
-                                "'%s' not in self._icmptypes" % obj.name)
+            raise FirewallError(errors.INVALID_ICMPTYPE, obj.name)
         if obj.path != config.ETC_FIREWALLD_ICMPTYPES:
             raise FirewallError(errors.INVALID_DIRECTORY,
                                 "'%s' != '%s'" % \
                                 (obj.path, config.ETC_FIREWALLD_ICMPTYPES))
-        os.remove("%s/%s.xml" % (obj.path, obj.name))
+
+        name = "%s/%s.xml" % (obj.path, obj.name)
+        try:
+            shutil.move(name, "%s.old" % name)
+        except Exception as msg:
+            log.error("Backup of file '%s' failed: %s", name, msg)
+            os.remove(name)
+
         del self._icmptypes[obj.name]
 
     def check_builtin_icmptype(self, obj):
@@ -521,7 +528,7 @@ class FirewallConfig(object):
                                 "self._services[%s] != obj" % obj.name)
         elif obj.name not in self._builtin_services:
             raise FirewallError(errors.NO_DEFAULTS,
-                            "'%s' not in self._builtin_services" % obj.name)
+                                "'%s' not a built-in service" % obj.name)
         self._remove_service(obj)
         return self._builtin_services[obj.name]
 
@@ -545,12 +552,9 @@ class FirewallConfig(object):
             return obj
 
     def new_service(self, name, conf):
-        try:
-            self.get_service(name)
-        except:
-            pass
-        else:
-            raise FirewallError(errors.NAME_CONFLICT, "new_service(): '%s'" % name)
+        if name in self._services:
+            raise FirewallError(errors.NAME_CONFLICT,
+                                "new_service(): '%s'" % name)
 
         x = Service()
         x.check_name(name)
@@ -637,13 +641,19 @@ class FirewallConfig(object):
 
     def _remove_service(self, obj):
         if obj.name not in self._services:
-            raise FirewallError(errors.INVALID_SERVICE,
-                                "'%s' not in self._services" % obj.name)
+            raise FirewallError(errors.INVALID_SERVICE, obj.name)
         if obj.path != config.ETC_FIREWALLD_SERVICES:
             raise FirewallError(errors.INVALID_DIRECTORY,
                                 "'%s' != '%s'" % \
                                 (obj.path, config.ETC_FIREWALLD_SERVICES))
-        os.remove("%s/%s.xml" % (obj.path, obj.name))
+
+        name = "%s/%s.xml" % (obj.path, obj.name)
+        try:
+            shutil.move(name, "%s.old" % name)
+        except Exception as msg:
+            log.error("Backup of file '%s' failed: %s", name, msg)
+            os.remove(name)
+
         del self._services[obj.name]
 
     def check_builtin_service(self, obj):
@@ -697,7 +707,7 @@ class FirewallConfig(object):
                                 "self._zones[%s] != obj" % obj.name)
         elif obj.name not in self._builtin_zones:
             raise FirewallError(errors.NO_DEFAULTS,
-                                "'%s' not in self._builtin_zones" % obj.name)
+                                "'%s' not a built-in zone" % obj.name)
         self._remove_zone(obj)
         return self._builtin_zones[obj.name]
 
@@ -723,11 +733,7 @@ class FirewallConfig(object):
             return obj
 
     def new_zone(self, name, conf):
-        try:
-            self.get_zone(name)
-        except:
-            pass
-        else:
+        if name in self._zones:
             raise FirewallError(errors.NAME_CONFLICT, "new_zone(): '%s'" % name)
 
         x = Zone()
@@ -824,13 +830,19 @@ class FirewallConfig(object):
 
     def _remove_zone(self, obj):
         if obj.name not in self._zones:
-            raise FirewallError(errors.INVALID_ZONE,
-                                "'%s' not in self._zones" % obj.name)
+            raise FirewallError(errors.INVALID_ZONE, obj.name)
         if not obj.path.startswith(config.ETC_FIREWALLD_ZONES):
             raise FirewallError(errors.INVALID_DIRECTORY,
                                 "'%s' doesn't start with '%s'" % \
                                 (obj.path, config.ETC_FIREWALLD_ZONES))
-        os.remove("%s/%s.xml" % (config.ETC_FIREWALLD_ZONES, obj.name))
+
+        name = "%s/%s.xml" % (obj.path, obj.name)
+        try:
+            shutil.move(name, "%s.old" % name)
+        except Exception as msg:
+            log.error("Backup of file '%s' failed: %s", name, msg)
+            os.remove(name)
+
         del self._zones[obj.name]
 
     def check_builtin_zone(self, obj):
